"""
1. Create UI
2. Compile UI as MainWindow, import it.
3. Read data from countries.txt, save it as countries.csv
4. Create a 2dlist out of the CSV values
5. Column 0 is country name, column 1 is country population and column 2 is country square miles.
   Create a menu that reads the CSV file the values from Column 0 in the different rows
6. Store data in memory.
7. Create labels/text boxes to display the country name, population and link it to the proper index.
8. Create a label for Flags and use the ID of the country name from the 2Dlist as a variable to get the
   name of the country. Create a variable and use it when calling the file path. Replacing spaces with underscores
9. Calculate conversion from square miles to square KM and display on screen when selected via radio button.
10. Calculate population percentage by taking the sum of all column 1 values and doing (current pop X 100) / sum of col 1
11. Enable save option in menu to save from menu to countries.txt
12. Add confirmation messages and save/exit confirmation.
Student Name:    Steven Crosby
Program Title:  Countries of the World
Description:    Final Project: Phase 3
"""

import sys

from PyQt5.QtWidgets import QApplication, QMainWindow, QMessageBox
from PyQt5.QtGui import QPixmap

#ADD IMPORT STATEMENT FOR YOUR GENERATED UI.PY FILE HERE
import Ui_FinalProject
#      ^^^^^^^^^^^ Change this!

#CHANGE THE SECOND PARAMETER (Ui_ChangeMe) TO MATCH YOUR GENERATED UI.PY FILE
class MyForm(QMainWindow, Ui_FinalProject.Ui_MainWindow):
#                         ^^^^^^^^^^^   Change this!
    def LoadCountriesFromFile(newCSVFile):   
        try:#load the file, convert to CSV and create a 2DLIST.
            fileName = "Files/countries.txt"
            accessMode = "r"    

            with open(fileName, accessMode) as myTXTFile:
                myTXT=(myTXTFile.read())

            fileName = "Files/countries.csv"
            accessMode = "w"    

            with open(fileName, accessMode) as newCSVFile:
                newCSVFile.write(myTXT)
        except FileNotFoundError:
            fileName = "Files/countries.txt"
            accessMode = "w"    

            with open(fileName, accessMode) as myTXTFile:
                data = "Error Reading from File Path,0,0"
                myTXTFile.write(data)
                myTXT = data

                fileName = "Files/countries.csv"
                accessMode = "w"    
            with open(fileName, accessMode) as newCSVFile:
                 newCSVFile.write(myTXT)
                 return myTXT
    newCSVFile = ""
    myTXT = LoadCountriesFromFile(newCSVFile)
    people = []
    my2dList = [] 
    unsaved_changes = False
    # DO NOT MODIFY THIS CODE
    def __init__(self, parent=None): #Runs when form loads, before user sees anything
        super(MyForm, self).__init__(parent)
        self.setupUi(self)
    # END DO NOT MODIFY
    

        # ADD SLOTS HERE, indented to this level (ie. inside def __init__)
        self.frameCountries.setVisible(False)
        self.listWidgetPeople.currentRowChanged.connect(self.DisplayCountryData)
        self.listWidgetPeople.currentRowChanged.connect(self.ShowImage)
        self.actionLoadCountries.triggered.connect(self.LoadCountriesListBox)
        self.actionSavetoFile.triggered.connect(self.saveNoExit)
        self.actionExit.triggered.connect(self.exitApplication)
        self.pushButtonUpdate.clicked.connect(self.updatePeople)
        self.comboBoxMiFt.currentIndexChanged.connect(self.CalculateTotalWorldPopulation)
        self.comboBoxMiFt.addItem("Sq. Miles")
        self.comboBoxMiFt.addItem("Sq. Feet")
        self.radioButtonMile.clicked.connect(self.radioButtonMileClicked)
        self.radioButtonKM.clicked.connect(self.radioButtonKMClicked)

    # ADD SLOT FUNCTIONS HERE
    # These are the functions your slots will point to
    # Indent to this level (ie. inside the class, at same level as def __init__)
    def LoadCountriesListBox(self):
        self.frameCountries.setVisible(True)
        if len(self.my2dList) == 0:
            self.LoadCountriesFromFile() 
        
        self.listWidgetPeople.clear()
        for row in self.my2dList:
            self.listWidgetPeople.addItem(row[0])
            #Add country names in 2Dlist to list widget
        self.sumList = []
        self.sumofSumList = []
        global compileList
        compileList = self.my2dList
        selectedCountryIndex = self.listWidgetPeople.currentRow()
        updatePop = (str(compileList[selectedCountryIndex][1]))
        try:#Check for errors in the list/if file is non existant
            updatePop = int(updatePop)
        except ValueError:
            QMessageBox.information(self, "Error", "Error reading from the file path. Closing program.", QMessageBox.Ok)
            exit()
        except:
            QMessageBox.information(self, "Error", "Error reading from the file path. Closing program.", QMessageBox.Ok)
            exit()

        self.lineEditPop.setText("{:,.0F}".format(updatePop))
        total = 0
        indexLoop = 0
        for row in compileList: 
            self.sumList.append(compileList[indexLoop][1])
            indexLoop = indexLoop + 1
        # appending the values of column 1 to its own list to sum
        for counter in range(0, len(self.sumList)):
            total = total + int(str(self.sumList[counter]))
            self.popTotal=str(total) 

    def CalculateTotalWorldPopulation(self):
        fileName = "Files/countries.csv"
        accessMode = "r"    

        with open(fileName, accessMode) as myCSVFile:
            import csv
            fileData = csv.reader(myCSVFile)
            
            self.my2dList = []
            for row in fileData:
                self.my2dList.append(row)
            comboIndex = self.comboBoxMiFt.currentIndex()
            try:
                if comboIndex == 1:
                    selectedCountryIndex = self.listWidgetPeople.currentRow()
                    convertKM1 = float(self.my2dList[selectedCountryIndex][2])
                    convertKM2 = float(2.58998811)
                    convertTotal= (convertKM1 * convertKM2)
                    self.labelEditSqm.setText("{:,.0F}".format(convertTotal))
                elif comboIndex == 0:
                    selectedCountryIndex = self.listWidgetPeople.currentRow()
                    convertTotal = float(self.my2dList[selectedCountryIndex][2])
                    self.labelEditSqm.setText("{:,.0F}".format(convertTotal))
            except ValueError:
                convertTotal = 1
                self.labelEditSqm.setText("{:,.0F}".format(convertTotal))
                #Square Mile to KM conversion + error checking

            

    def DisplayCountryData(self, selectedCountryIndex):
        OneHundred = 100
        convertPop=(compileList[selectedCountryIndex][1])
        try:
            convertPop = float(convertPop)
            self.lineEditPop.clear()
            self.lineEditPop.setText("{:,.0F}".format(convertPop))
            self.lineEditPop.update()
        except ValueError:
            value = self.listWidgetPeople.currentItem() 
            if value == "Error Reading from File Path":
                QMessageBox.information(self, "Invalid", "Data is invalid so not updated in memory", QMessageBox.Ok)
                exit()
            else:
                QMessageBox.information(self, "Invalid", "Data is invalid so not updated in memory", QMessageBox.Ok)
        try:
            popLabel = (convertPop)/float(self.popTotal)*float(OneHundred)
        except AttributeError:
            exit()
        except TypeError:
            QMessageBox.information(self, "Invalid", "Data is invalid so not updated in memory", QMessageBox.Ok)
        except ZeroDivisionError:
            QMessageBox.information(self, "Error", "Error reading from the file path. Closing program.", QMessageBox.Ok)
            exit()
        self.labelCountry.clear()
        self.labelCountry.setText(str(compileList[selectedCountryIndex][0]))
        self.labelCountry.update()
        self.labelEditSqm.clear()
        MiFeetUpdate= (compileList[selectedCountryIndex][2])
        try:
            MiFeetUpdate = float(MiFeetUpdate)
        except ValueError:
            MiFeetUpdate = 0.0
            QMessageBox.information(self, "Error", "Error reading from the file path. Closing program.", QMessageBox.Ok)
            exit()
        self.labelEditSqm.setText("{:,.1F}".format(MiFeetUpdate))
        self.labelEditSqm.update()
        MiFeetUpdate = str(MiFeetUpdate)
        MiFeetUpdate = MiFeetUpdate.replace(",","_")
        MiFeetUpdate = MiFeetUpdate.replace("_","")
        self.labelEditPopulation.clear()
        try:
            self.labelEditPopulation.setText("{:.4f}%".format(popLabel))
            self.labelEditPopulation.update()
        except UnboundLocalError:
            QMessageBox.information(self, "Invalid", "Data is invalid so not updated in memory", QMessageBox.Ok)
            #population percentage calculations and program \
            # refresh after value update


    def radioButtonMileClicked(self, enabled):
        selectedCountryIndex = self.listWidgetPeople.currentRow()
        squareMi1 = float(compileList[selectedCountryIndex][1])
        squareMi2 = float(compileList[selectedCountryIndex][2])
        squareMiTotal= (squareMi1) / (squareMi2)
        if enabled:
            self.comboBoxMiFt.currentIndexChanged.connect(self.populateListWithPeople)
            self.labelEditDensity.clear()
            self.labelEditDensity.setText("{:.2f}".format(squareMiTotal))
            self.listWidgetPeople.currentRowChanged.connect(self.radioButtonMileClicked)
            self.listWidgetPeople.currentRowChanged.connect(self.radioButtonKMClicked)
        #Radio button conversion
    def radioButtonKMClicked(self, enabled):
        selectedCountryIndex = self.listWidgetPeople.currentRow()
        squareKM1 = float(compileList[selectedCountryIndex][1])
        squareKM2 = float(2.58998811)
        squareKM3 = float(compileList[selectedCountryIndex][2])
        squareKMTotal= (squareKM1 * squareKM2) / (squareKM3)
        if enabled:
            self.labelEditDensity.clear()
            self.labelEditDensity.setText("{:.2f}".format(squareKMTotal))
            self.listWidgetPeople.currentRowChanged.connect(self.radioButtonMileClicked)
            self.listWidgetPeople.currentRowChanged.connect(self.radioButtonKMClicked)

    #ADD HELPER FUNCTIONS HERE
    # These are the functions the slot functions will call, to 
    # contain the custom code that you'll write to make your progam work.
    # Indent to this level (ie. inside the class, at same level as def __init__)

    def ShowImage(self):
        fileName = "Files/countries.csv"
        accessMode = "r"
        with open(fileName, accessMode) as imageCSVFile:
            import csv
            imageData = csv.reader(imageCSVFile)    #KIND OF gives me a 2d list of the file's contents
            
            self.imageList = []
            for row in imageData:
                self.imageList.append(row)
        index = self.listWidgetPeople.currentRow()
        printIndex=(self.imageList[index][0])
        printIndex = printIndex.replace(" ", "_")
        image=QPixmap("Files/Flags/"+str(printIndex)+".png")
        self.labelPicture.clear()
        self.labelPicture.setPixmap(image)
        #get proper flag image loaded

    def closeEvent(self, event):
        self.unsaved_changes == True
        quit_msg = "Save changes to file before closing?"
        reply = QMessageBox.question(self, 'Save?',
                quit_msg, QMessageBox.Yes, QMessageBox.No)

        if reply == QMessageBox.Yes:
            self.unsaved_changes == False
            self.actionSavetoFile.setEnabled(False)
            myFile=""
            self.SaveCountriesToFile(myFile)
            QMessageBox.information(self, 'Saved', 'Changes were saved to the file', QMessageBox.Ok)
        else:
            exit()
            #Closing (click the X)

    def saveNoExit(self, event):
        self.unsaved_changes == True
        quit_msg = "Save changes to file?"
        reply = QMessageBox.question(self, 'Save?',
                quit_msg, QMessageBox.Yes, QMessageBox.No)
        if reply == QMessageBox.Yes:
            myFile=""
            self.SaveCountriesToFile(myFile)
            self.actionSavetoFile.setEnabled(False)
        if reply == QMessageBox.No:
            self.unsaved_changes == False
            pass
        #(Save menu)

    def exitApplication(self, event):
        self.unsaved_changes == True
        quit_msg = "Save changes to file before closing?"
        reply = QMessageBox.question(self, 'Save?',
                quit_msg, QMessageBox.Yes, QMessageBox.No)
        if reply == QMessageBox.Yes:
            myFile=""
            self.SaveCountriesToFile(myFile)
            exit()
        if reply == QMessageBox.No:
            self.unsaved_changes == False
            exit()
            pass
        # File > Exit
            
    def SaveCountriesToFile(self, myFile):
        with open("Files/countries.txt", "w") as myFile:
            for person in compileList:
                finalPrint = person[1].replace("_","")
                selected_index = self.listWidgetPeople.currentRow()
                finalPrint=compileList[selected_index][1]
                finalPrint = self.lineEditPop.text()
                myFile.write(",".join(person) + "\n")
                self.unsaved_changes == False
                #Saving function

    def updatePeople(self):
        selected_index = self.listWidgetPeople.currentRow()      
        replaceUnderscore = self.lineEditPop.text()        
        replaceUnderscore=replaceUnderscore.replace(",","_")
        replaceUnderscore=replaceUnderscore.replace("_","")
        if replaceUnderscore.isalpha() != True:
            self.lineEditPop.setText(replaceUnderscore)
            (compileList[selected_index][1]) = self.lineEditPop.text()
            replaceUnderscore=int(replaceUnderscore)
            self.lineEditPop.setText("{:,.0F}".format(replaceUnderscore))
            QMessageBox.information(self, 'Updated', "Data has been updated in memory, but hasn't been updated in the file yet.", QMessageBox.Ok)
            self.actionSavetoFile.setEnabled(True)
            self.unsaved_changes == True
            self.populateListWithPeople()
            replaceUnderscore=str(replaceUnderscore)
        if replaceUnderscore.isalpha() == True:
            try:
                self.unsaved_changes == False
                replaceUnderscore=str(replaceUnderscore)
                self.lineEditPop.setText("{:,.0F}".format(replaceUnderscore))
            except ValueError:
                QMessageBox.information(self, "Invalid", "Data is invalid so not updated in memory", QMessageBox.Ok)       
                self.lineEditPop.setText(self.my2dList[selected_index][1])
                (compileList[selected_index][1]) = self.lineEditPop.text()
                replaceUnderscore = (compileList[selected_index][1])
                replaceUnderscore=int(replaceUnderscore)
                self.lineEditPop.setText("{:,.0F}".format(replaceUnderscore))           
                #Error checking + removing underscore from user inputs


    def populateListWithPeople(self): 
        self.unsaved_changes == True
        self.lineEditPop.clear()
        selected_index = self.listWidgetPeople.currentRow()
        strCheck = (compileList[selected_index][1])
        replaceUnderscore = strCheck        
        replaceUnderscore=replaceUnderscore.replace(",","_")
        replaceUnderscore=replaceUnderscore.replace("_","")
        intStrCheck = int(replaceUnderscore)
        self.lineEditPop.clear()
        self.lineEditPop.setText("{:,.0F}".format(intStrCheck))
        #repopulate list, check there are no underscores

# DO NOT MODIFY THIS CODE
if __name__ == "__main__":
    app = QApplication(sys.argv)
    the_form = MyForm()
    the_form.show()
    sys.exit(app.exec_())
# END DO NOT MODIFY